package sortie.data.funcgroups.mortality;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import sortie.data.funcgroups.Behavior;
import sortie.data.funcgroups.MortalityBehaviors;
import sortie.data.simpletypes.ModelException;
import sortie.gui.GUIManager;
import junit.framework.TestCase;

public class SenescenceMortTest extends TestCase {
  
  public void testReadV6ParFile() {
    GUIManager oManager = null;
    String sFileName = null;
    String sNewFileName = "c:\\test7.xml";
    try {

      oManager = new GUIManager(null);

      oManager.clearCurrentData();
      sFileName = write6XMLValidFile();
      oManager.convertPre7ParameterFile(sFileName, sNewFileName);
      oManager.inputXMLParameterFile(sNewFileName);
      MortalityBehaviors oMortBeh = oManager.getMortalityBehaviors();
      ArrayList<Behavior> p_oBehs = oMortBeh.getBehaviorByParameterFileTag("Senescence");
      assertEquals(1, p_oBehs.size());
      SenescenceMort oMort = (SenescenceMort) p_oBehs.get(0);
      
      assertEquals(-5.0, ((Float)oMort.mp_fRandomMortalityAlpha.getValue().get(0)).floatValue(), 0.0001);
      assertEquals(-6.0, ((Float)oMort.mp_fRandomMortalityAlpha.getValue().get(1)).floatValue(), 0.0001);
      assertEquals(-7.0, ((Float)oMort.mp_fRandomMortalityAlpha.getValue().get(2)).floatValue(), 0.0001);
      assertEquals(-8.0, ((Float)oMort.mp_fRandomMortalityAlpha.getValue().get(3)).floatValue(), 0.0001);

      assertEquals(0.135, ((Float)oMort.mp_fRandomMortalityBeta.getValue().get(0)).floatValue(), 0.0001);
      assertEquals(0.136, ((Float)oMort.mp_fRandomMortalityBeta.getValue().get(1)).floatValue(), 0.0001);
      assertEquals(0.137, ((Float)oMort.mp_fRandomMortalityBeta.getValue().get(2)).floatValue(), 0.0001);
      assertEquals(0.138, ((Float)oMort.mp_fRandomMortalityBeta.getValue().get(3)).floatValue(), 0.0001);
      
      assertEquals(50.0, ((Float)oMort.mp_fDbhAtOnsetOfSenescence.getValue().get(0)).floatValue(), 0.0001);
      assertEquals(51.0, ((Float)oMort.mp_fDbhAtOnsetOfSenescence.getValue().get(1)).floatValue(), 0.0001);
      assertEquals(52.0, ((Float)oMort.mp_fDbhAtOnsetOfSenescence.getValue().get(2)).floatValue(), 0.0001);
      assertEquals(53.0, ((Float)oMort.mp_fDbhAtOnsetOfSenescence.getValue().get(3)).floatValue(), 0.0001);
      
      assertEquals(110, oMort.m_iDbhAtAsymptoticMaximumMortality.getValue());
    }
    catch (ModelException oErr) {
      fail("Parameter file read failed with message " + oErr.getMessage());
    }
    catch (IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    } finally {
      new File(sNewFileName).delete();
      new File(sFileName).delete();
    }
  }
  
  public void testReadParFile() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);

      oManager.clearCurrentData();
      sFileName = writeValidXMLFile();
      oManager.inputXMLParameterFile(sFileName);
      MortalityBehaviors oMortBeh = oManager.getMortalityBehaviors();
      ArrayList<Behavior> p_oBehs = oMortBeh.getBehaviorByParameterFileTag("Senescence");
      assertEquals(1, p_oBehs.size());
      SenescenceMort oMort = (SenescenceMort) p_oBehs.get(0);

      assertEquals(-8.0, ((Float)oMort.mp_fRandomMortalityAlpha.getValue().get(1)).floatValue(), 0.0001);
      assertEquals(-9.0, ((Float)oMort.mp_fRandomMortalityAlpha.getValue().get(2)).floatValue(), 0.0001);
      assertEquals(-10.0, ((Float)oMort.mp_fRandomMortalityAlpha.getValue().get(3)).floatValue(), 0.0001);
      assertEquals(-11.0, ((Float)oMort.mp_fRandomMortalityAlpha.getValue().get(4)).floatValue(), 0.0001);

      assertEquals(0.135, ((Float)oMort.mp_fRandomMortalityBeta.getValue().get(1)).floatValue(), 0.0001);
      assertEquals(0.235, ((Float)oMort.mp_fRandomMortalityBeta.getValue().get(2)).floatValue(), 0.0001);
      assertEquals(0.335, ((Float)oMort.mp_fRandomMortalityBeta.getValue().get(3)).floatValue(), 0.0001);
      assertEquals(0.435, ((Float)oMort.mp_fRandomMortalityBeta.getValue().get(4)).floatValue(), 0.0001);
      
      assertEquals(50.0, ((Float)oMort.mp_fDbhAtOnsetOfSenescence.getValue().get(1)).floatValue(), 0.0001);
      assertEquals(60.0, ((Float)oMort.mp_fDbhAtOnsetOfSenescence.getValue().get(2)).floatValue(), 0.0001);
      assertEquals(70.0, ((Float)oMort.mp_fDbhAtOnsetOfSenescence.getValue().get(3)).floatValue(), 0.0001);
      assertEquals(80.0, ((Float)oMort.mp_fDbhAtOnsetOfSenescence.getValue().get(4)).floatValue(), 0.0001);
      
      assertEquals(110, oMort.m_iDbhAtAsymptoticMaximumMortality.getValue());
                  
    }
    catch (ModelException oErr) {
      fail("Parameter file read failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }


  /**
   * Writes a file with valid settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  private String writeValidXMLFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);
    
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"07010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>2</timesteps>");
    oOut.write("<rt_timestep>0</rt_timestep>");
    oOut.write("<randomSeed>-1</randomSeed>");
    oOut.write("<yearsPerTimestep>1.0</yearsPerTimestep>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_precip_mm_yr>1150.645781</plot_precip_mm_yr>");
    oOut.write("<plot_temp_C>12.88171785</plot_temp_C>");
    oOut.write("<plot_latitude>0.0</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("<tr_species speciesName=\"Species_3\" />");
    oOut.write("<tr_species speciesName=\"Species_4\" />");
    oOut.write("<tr_species speciesName=\"Species_5\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_3\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_4\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_5\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_maxSeedlingHeight>");
    oOut.write("<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"Species_3\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"Species_4\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"Species_5\">1.35</tr_mshVal>");
    oOut.write("</tr_maxSeedlingHeight>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">40.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">40.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_3\">40.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_4\">40.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_5\">40.0</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.7</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.7</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_3\">0.7</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_4\">0.7</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_5\">0.7</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_3\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_4\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_5\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.7</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.7</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_3\">0.7</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_4\">0.7</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_5\">0.7</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_interceptDiam10ToDBH>");
    oOut.write("<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"Species_3\">0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"Species_4\">0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"Species_5\">0</tr_idtdVal>");
    oOut.write("</tr_interceptDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.7</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.7</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_3\">0.7</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_4\">0.7</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_5\">0.7</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_4\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_5\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_3\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_4\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_5\">0.03</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.7</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.7</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_3\">0.7</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_4\">0.7</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_5\">0.7</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("<tr_whatSeedlingHeightDiam>");
    oOut.write("<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"Species_3\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"Species_4\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"Species_5\">0</tr_wsehdVal>");
    oOut.write("</tr_whatSeedlingHeightDiam>");
    oOut.write("<tr_whatSaplingHeightDiam>");
    oOut.write("<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"Species_3\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"Species_4\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"Species_5\">0</tr_wsahdVal>");
    oOut.write("</tr_whatSaplingHeightDiam>");
    oOut.write("<tr_whatAdultHeightDiam>");
    oOut.write("<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"Species_3\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"Species_4\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"Species_5\">0</tr_wahdVal>");
    oOut.write("</tr_whatAdultHeightDiam>");
    oOut.write("<tr_whatAdultCrownRadDiam>");
    oOut.write("<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"Species_3\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"Species_4\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"Species_5\">0</tr_wacrdVal>");
    oOut.write("</tr_whatAdultCrownRadDiam>");
    oOut.write("<tr_whatAdultCrownHeightHeight>");
    oOut.write("<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"Species_3\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"Species_4\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"Species_5\">0</tr_wachhVal>");
    oOut.write("</tr_whatAdultCrownHeightHeight>");
    oOut.write("<tr_whatSaplingCrownRadDiam>");
    oOut.write("<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"Species_3\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"Species_4\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"Species_5\">0</tr_wscrdVal>");
    oOut.write("</tr_whatSaplingCrownRadDiam>");
    oOut.write("<tr_whatSaplingCrownHeightHeight>");
    oOut.write("<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"Species_3\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"Species_4\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"Species_5\">0</tr_wschhVal>");
    oOut.write("</tr_whatSaplingCrownHeightHeight>");
    oOut.write("</allometry>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>ConstRadialGrowth</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<listPosition>1</listPosition>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_4\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_5\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_4\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_5\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_4\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_5\" type=\"Adult\" />");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Senescence</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<listPosition>2</listPosition>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_4\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_5\" type=\"Adult\" />");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>RemoveDead</behaviorName>");
    oOut.write("<version>2.0</version>");
    oOut.write("<listPosition>3</listPosition>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_4\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_4\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_4\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_5\" type=\"Adult\" />");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<ConstRadialGrowth1>");
    oOut.write("<gr_adultConstRadialInc>");
    oOut.write("<gr_acriVal species=\"Species_3\">0.0</gr_acriVal>");
    oOut.write("<gr_acriVal species=\"Species_4\">0.0</gr_acriVal>");
    oOut.write("<gr_acriVal species=\"Species_1\">0.0</gr_acriVal>");
    oOut.write("<gr_acriVal species=\"Species_2\">0.0</gr_acriVal>");
    oOut.write("<gr_acriVal species=\"Species_5\">0.0</gr_acriVal>");
    oOut.write("</gr_adultConstRadialInc>");
    oOut.write("</ConstRadialGrowth1>");
    oOut.write("<Senescence2>");
    oOut.write("<mo_senescenceAlpha>");
    oOut.write("<mo_saVal species=\"Species_2\">-8.0</mo_saVal>");
    oOut.write("<mo_saVal species=\"Species_3\">-9.0</mo_saVal>");
    oOut.write("<mo_saVal species=\"Species_4\">-10.0</mo_saVal>");
    oOut.write("<mo_saVal species=\"Species_5\">-11.0</mo_saVal>");
    oOut.write("</mo_senescenceAlpha>");
    oOut.write("<mo_senescenceBeta>");
    oOut.write("<mo_sbVal species=\"Species_2\">0.135</mo_sbVal>");
    oOut.write("<mo_sbVal species=\"Species_3\">0.235</mo_sbVal>");
    oOut.write("<mo_sbVal species=\"Species_4\">0.335</mo_sbVal>");
    oOut.write("<mo_sbVal species=\"Species_5\">0.435</mo_sbVal>");
    oOut.write("</mo_senescenceBeta>");
    oOut.write("<mo_dbhAtOnsetOfSenescence>");
    oOut.write("<mo_daoosVal species=\"Species_2\">50.0</mo_daoosVal>");
    oOut.write("<mo_daoosVal species=\"Species_3\">60.0</mo_daoosVal>");
    oOut.write("<mo_daoosVal species=\"Species_4\">70.0</mo_daoosVal>");
    oOut.write("<mo_daoosVal species=\"Species_5\">80.0</mo_daoosVal>");
    oOut.write("</mo_dbhAtOnsetOfSenescence>");
    oOut.write("<mo_dbhAtAsympMaxMort>110</mo_dbhAtAsympMaxMort>");
    oOut.write("</Senescence2>");
    oOut.write("</paramFile>");
    
    oOut.close();

    return sFileName;
  }
  
  /**
   * Writes a valid version 6 file.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  private String write6XMLValidFile() throws IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>2</timesteps>");
    oOut.write("<rt_timestep>0</rt_timestep>");
    oOut.write("<randomSeed>-1</randomSeed>");
    oOut.write("<yearsPerTimestep>5.0</yearsPerTimestep>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_precip_mm_yr>1150.645781</plot_precip_mm_yr>");
    oOut.write("<plot_temp_C>12.88171785</plot_temp_C>");
    oOut.write("<plot_latitude>0.0</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("<tr_species speciesName=\"Species_3\" />");
    oOut.write("<tr_species speciesName=\"Species_4\" />");
    oOut.write("<tr_species speciesName=\"Species_5\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_3\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_4\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_5\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_maxSeedlingHeight>");
    oOut.write("<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"Species_3\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"Species_4\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"Species_5\">1.35</tr_mshVal>");
    oOut.write("</tr_maxSeedlingHeight>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">40.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">40.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_3\">40.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_4\">40.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_5\">40.0</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.7</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.7</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_3\">0.7</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_4\">0.7</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_5\">0.7</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_3\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_4\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_5\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.7</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.7</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_3\">0.7</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_4\">0.7</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_5\">0.7</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_interceptDiam10ToDBH>");
    oOut.write("<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"Species_3\">0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"Species_4\">0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"Species_5\">0</tr_idtdVal>");
    oOut.write("</tr_interceptDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.7</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.7</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_3\">0.7</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_4\">0.7</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_5\">0.7</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_4\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_5\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_3\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_4\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_5\">0.03</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.7</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.7</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_3\">0.7</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_4\">0.7</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_5\">0.7</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("<tr_whatSeedlingHeightDiam>");
    oOut.write("<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"Species_3\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"Species_4\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"Species_5\">0</tr_wsehdVal>");
    oOut.write("</tr_whatSeedlingHeightDiam>");
    oOut.write("<tr_whatSaplingHeightDiam>");
    oOut.write("<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"Species_3\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"Species_4\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"Species_5\">0</tr_wsahdVal>");
    oOut.write("</tr_whatSaplingHeightDiam>");
    oOut.write("<tr_whatAdultHeightDiam>");
    oOut.write("<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"Species_3\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"Species_4\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"Species_5\">0</tr_wahdVal>");
    oOut.write("</tr_whatAdultHeightDiam>");
    oOut.write("<tr_whatAdultCrownRadDiam>");
    oOut.write("<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"Species_3\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"Species_4\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"Species_5\">0</tr_wacrdVal>");
    oOut.write("</tr_whatAdultCrownRadDiam>");
    oOut.write("<tr_whatAdultCrownHeightHeight>");
    oOut.write("<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"Species_3\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"Species_4\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"Species_5\">0</tr_wachhVal>");
    oOut.write("</tr_whatAdultCrownHeightHeight>");
    oOut.write("<tr_whatSaplingCrownRadDiam>");
    oOut.write("<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"Species_3\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"Species_4\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"Species_5\">0</tr_wscrdVal>");
    oOut.write("</tr_whatSaplingCrownRadDiam>");
    oOut.write("<tr_whatSaplingCrownHeightHeight>");
    oOut.write("<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"Species_3\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"Species_4\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"Species_5\">0</tr_wschhVal>");
    oOut.write("</tr_whatSaplingCrownHeightHeight>");
    oOut.write("</allometry>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>constradialgrowth</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_4\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_5\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_4\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_5\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_4\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_5\" type=\"Adult\" />");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>bcmortality</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\" />");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>gmfmortality</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\" />");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>adultselfthin</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_5\" type=\"Adult\" />");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>juvselfthin</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_4\" type=\"Sapling\" />");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>senescence</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_4\" type=\"Adult\" />");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>adultstochasticmort</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\" />");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>juvstochasticmort</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_4\" type=\"Seedling\" />");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>removedead</behaviorName>");
    oOut.write("<version>2.0</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_4\" type=\"Seedling\" />");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_4\" type=\"Adult\" />");
    oOut.write("<applyTo species=\"Species_4\" type=\"Sapling\" />");
    oOut.write("<applyTo species=\"Species_5\" type=\"Adult\" />");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<growth>");
    oOut.write("<gr_adultConstRadialInc>");
    oOut.write("<gr_acriVal species=\"Species_3\">0.0</gr_acriVal>");
    oOut.write("<gr_acriVal species=\"Species_4\">0.0</gr_acriVal>");
    oOut.write("<gr_acriVal species=\"Species_1\">0.0</gr_acriVal>");
    oOut.write("<gr_acriVal species=\"Species_2\">0.0</gr_acriVal>");
    oOut.write("<gr_acriVal species=\"Species_5\">0.0</gr_acriVal>");
    oOut.write("</gr_adultConstRadialInc>");
    oOut.write("</growth>");
    oOut.write("<mortality>");
    oOut.write("<mo_randomJuvenileMortality>");
    oOut.write("<mo_rjmVal species=\"Species_3\">0.3</mo_rjmVal>");
    oOut.write("<mo_rjmVal species=\"Species_4\">0.0</mo_rjmVal>");
    oOut.write("</mo_randomJuvenileMortality>");
    oOut.write("<mo_nonSizeDepMort>");
    oOut.write("<mo_nsdmVal species=\"Species_3\">1.0</mo_nsdmVal>");
    oOut.write("</mo_nonSizeDepMort>");
    oOut.write("<mo_mortAtZeroGrowth>");
    oOut.write("<mo_mazgVal species=\"Species_1\">1.0</mo_mazgVal>");
    oOut.write("<mo_mazgVal species=\"Species_2\">0.99</mo_mazgVal>");
    oOut.write("</mo_mortAtZeroGrowth>");
    oOut.write("<mo_lightDependentMortality>");
    oOut.write("<mo_ldmVal species=\"Species_1\">5.2</mo_ldmVal>");
    oOut.write("<mo_ldmVal species=\"Species_2\">6.64</mo_ldmVal>");
    oOut.write("</mo_lightDependentMortality>");
    oOut.write("<mo_juvSelfThinSlope>");
    oOut.write("<mo_jstsVal species=\"Species_4\">0.02</mo_jstsVal>");
    oOut.write("</mo_juvSelfThinSlope>");
    oOut.write("<mo_juvSelfThinIntercept>");
    oOut.write("<mo_jstiVal species=\"Species_4\">0.01</mo_jstiVal>");
    oOut.write("</mo_juvSelfThinIntercept>");
    oOut.write("<mo_adultSelfThinSlope>");
    oOut.write("<mo_astsVal species=\"Species_5\">0.02</mo_astsVal>");
    oOut.write("</mo_adultSelfThinSlope>");
    oOut.write("<mo_adultSelfThinIntercept>");
    oOut.write("<mo_astiVal species=\"Species_5\">0.01</mo_astiVal>");
    oOut.write("</mo_adultSelfThinIntercept>");
    oOut.write("<mo_adultSelfThinMaxDbh>");
    oOut.write("<mo_astmdVal species=\"Species_5\">15.0</mo_astmdVal>");
    oOut.write("</mo_adultSelfThinMaxDbh>");
    oOut.write("<mo_randomMortAlpha>");
    oOut.write("<mo_rmaVal species=\"Species_1\">-5.0</mo_rmaVal>");
    oOut.write("<mo_rmaVal species=\"Species_2\">-6.0</mo_rmaVal>");
    oOut.write("<mo_rmaVal species=\"Species_3\">-7.0</mo_rmaVal>");
    oOut.write("<mo_rmaVal species=\"Species_4\">-8.0</mo_rmaVal>");
    oOut.write("</mo_randomMortAlpha>");
    oOut.write("<mo_randomMortBeta>");
    oOut.write("<mo_rmbVal species=\"Species_1\">0.135</mo_rmbVal>");
    oOut.write("<mo_rmbVal species=\"Species_2\">0.136</mo_rmbVal>");
    oOut.write("<mo_rmbVal species=\"Species_3\">0.137</mo_rmbVal>");
    oOut.write("<mo_rmbVal species=\"Species_4\">0.138</mo_rmbVal>");
    oOut.write("</mo_randomMortBeta>");
    oOut.write("<mo_dbhAtOnsetOfSenescence>");
    oOut.write("<mo_daoosVal species=\"Species_1\">50.0</mo_daoosVal>");
    oOut.write("<mo_daoosVal species=\"Species_2\">51.0</mo_daoosVal>");
    oOut.write("<mo_daoosVal species=\"Species_3\">52.0</mo_daoosVal>");
    oOut.write("<mo_daoosVal species=\"Species_4\">53.0</mo_daoosVal>");
    oOut.write("</mo_dbhAtOnsetOfSenescence>");
    oOut.write("<mo_dbhAtAsympMaxMort>110</mo_dbhAtAsympMaxMort>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");
    
    oOut.close();
    return sFileName;
  }
}
